// ===========Oooo==========================================Oooo========
// =  Copyright (C) 2014 Jinan Gowin Semiconductor Technology Co.,Ltd.
// =                     All rights reserved.
// =====================================================================
//
//  __      __      __
//  \ \    /  \    / /   [File name   ] intface.v
//   \ \  / /\ \  / /    [Description ] Verilog File for UART Controller Interface Module base on AXI-Lite
//    \ \/ /  \ \/ /     [Timestamp   ] Tue Aug 27 15:30:00 2015
//     \  /    \  /      [version     ] 1.0
//      \/      \/       
// --------------------------------------------------------------------
// Code Revision History :
// --------------------------------------------------------------------
// Ver: | Author |Mod. Date |Changes Made:
// V1.0 | XX     |27/08/15  |Initial version
// ===========Oooo==========================================Oooo========
//// ////////////////////////////////////////////////////////////////////
////
////     <Global reset & clock>
////      Reset       : Master reset
////      Clk16X      : UART internal clock
////
////    <Process| interface>
////      A           : Address bus
////      DIN         : Data bus input
////      DOUT        : Data but output
////      DDIS        : Driver disable
////      INTR        : Interrupt
////
////    <Registers>
////      RBR         : Receiver Buffer Register
////      THR         : Transmitter Holding Register
////      MSR         : Modem Status Register
////      MCR         : Modem Control Register
////
////    <Rising edge of registers read/write strobes>
////      RbrRDn_re   : one Clk16X width pulse indicating rising edge of RbrRDn_r
////      ThrWRn_re   : one Clk16X width pulse indicating rising edge of ThrWRn_r
////      LsrRDn_re   : one Clk16X width pulse indicating rising edge of LsrRDn_r
////      MsrRDn_re   : one Clk16X width pulse indicating rising edge of MsrRDn_r
////
////    <Receiver/Transmitter control>
////      Databits    : "00"=5-bit, "01"=6-bit, "10"=7-bit, "11"=8-bit
////      Stopbits    : "00"=1-bit, "01"=1.5-bit(5-bit data),
////                    "10"=2-bit(6,7,8-bit data)
////      ParityEnable: 1'b0=Parity Bit Enable, 1'b1=Parity Bit Disable
////      ParityEven  : 1'b0=Even Parity Selected, 1'b1=Odd Parity Selected
////      ParityStick : 1'b0=Stick Parity Disable, 1'b1=Stick Parity Enable
////      TxBreak     : 1'b0=Disable BREAK assertion, 1'b1=Assert BREAK
////
////    <Receiver/Transmitter status>
////      RxRDY       : RBR data is ready to be read
////      OverrunErr  : Overrun err|
////      ParityErr   : Parity err|
////      FrameErr    : Frame err|
////      BreakInt    : BREAK interrupt
////      THRE        : THR is empty
////      TEMT        : Both THR & TSR are empty
////
//// ////////////////////////////////////////////////////////////////////////////////
////   GENERAL REGISTER:                                                      //
////////////////////////////                                                  //
////   =================================================================      //
////  |  ADDRESS A[2:0]   |            REGISTER             | IMPLEMENT |     //
////   =================================================================      //
////  | $000 (READ)       | RBR (RECEIVER BUFFER REGISTER)  |  Y        |     //
////   ////////////////////////////////////////////////////////////////-      //
////  | $000 (WRITE)      | THR (TRANSMIT HOLD REGISTER)    |  Y        |     //
////   =================================================================      //
////  | $001 (WRITE)      | IER (INTERRUPT ENABLE REGISTER) |  Y        |     //
////   =================================================================      //
////  | $010 (READ)       | IIR (INTERRUPT ID REGISTER)     |  Y        |     //
////   =================================================================      //
////  | $011 (WRITE)      | LCR (LINE CONTROL REGISTER)     |  Y        |     //
////   =================================================================      //
////  | $100 (WRITE)      | MCR (MODEM CONTROL REGISTER)    |  Y        |     //
////   =================================================================      //
////  | $101 (READ)       | LSR (LINE STATUS REGISTER)      |  Y        |     //
////   =================================================================      //
////  | $110 (READ)       | MSR (MODEM STATUS REGISTER)     |  Y        |     //
////   =================================================================      //
////  | $111 (READ/WRITE) | SCR (SCRATCHPAD REGISTER)       |  N        |     //
////   =================================================================      //
////                                                                          //
////  ~E:  By using Lattice ISP solution, the Baud Rate can be
////         re-configured even when the device is soldered on the board.     //
////         Therefore the Baud Rate register set is omitted.                 //
////                                                                          //
////         Because each Lattice ispLSI device has a embedded UES register,  //
////         the Scratchpad register can be omitted too.                      //
////                                                                          //
////////////////////////////////////////////////////////////////////////////////
////  REGISTER BIT FIELDS:                                                    //
//////////////////////////////                                                //
////                                                                          //
////    ============================                                          //
////   | LSR (LINE STATUS REGISTER) |                                         //
////    ==============================================================        //
////   |  0    | TEMT | THRE  |  BI   |  FE   |  PE   |  OE   | RxRDY |       //
////    ==============================================================        //
////                                                                          //
////    RxRDY : RECEIVE DATA READY                                            //
////    OE    : OVERRUN ERR|                                                  //
////    PE    : PARITY ERR|                                                   //
////    FE    : FRAMING ERR|                                                  //
////    BI    : BREAK INTERRUPT                                               //
////    THRE  : TRASMITTER HOLDING REGISTER EMPTY                             //
////    TEMT  : TRASMITTER EMPTY                                              //
////                                                                          //
////                                                                          //
////    RxRDY: The data received flag is set to 1 at the successful           //
////           completion of a byte receive cycle.  It is automatically       //
////           cleared to 0 when the Rx Data Register is read.  If a new byte //
////           is received before an Rx Data Register read, the over run flag //
////           will be set to 1. If (SR) status option is set the UART will   //
////           ign|e all further incoming bytes until the Rx Data Register    //
////           has been read.                                                 //
////                                                                          //
////    OE:    It indicates that the data in RBR was ~ read by the CPU        //
////           before the next character arrived, thereby destroying the the  //
////           previous character. The OE indicat| is set to 1 upon           //
////           detection of an overrun condition & reset whenever the CPU     //
////           reads the contents of LSR                                      //
////                                                                          //
////    PE:    The parity err| flag is set to 1 if an invalid parity bit is  //
////           encountered. It is automatically cleared to 0 when the CPU     //
////           reads the contents of Line Status Register.                    //
////                                                                          //
////    FE:    The framing err| flag is set to 1 if an invalid stop bit is   //
////           encountered. It is automatically cleared to 0 when the CPU     //
////           reads the contents of Line Status Register.                    //
////                                                                          //
////    BI:    The start bit err| flag is set to 1 if an invalid start bit   //
////           is encountered. It is automatically cleared to 0 when the CPU  //
////           reads the contents of Line Status Register.                    //
////                                                                          //
////    THRE:  The Transmit Holding Register Empty flag indicate that the     //
////           UART is ready to accept a new character for transmission. In   //
////           addition this bit cause the UART issue an interrupt to the CPU //
////           when the THRE interrupt enable is set to high                  //
////                                                                          //
////    TEMT:  The Transmitter Empty indicat| is set to 1'b1 whenever         //
////           whenever the Transmitter Holding Register & the Transmitter  //
////           Shifting Register are both empty. It is reset to 1'b0 whenever  //
////           either the Transmitter Holding register | Transmitter Shift   //
////           Register contains a character                                  //
////                                                                          //
////    ================================                                      //
////   | LCR (LINE CONTROL REGISTER)    |                                     //
////    ===============================================================       //
////   |  DLAB |  SB   |  SP   | EPS   |  PEN  |  STB  | WLS1  | WLS0  |      //
////    ===============================================================       //
////                                                                          //
////    WLS1-WLS0:  W|D LENGTH SELECT  00 = 5 DATA BITS                      //
////                                    01 = 6 DATA BITS                      //
////                                    10 = 7 DATA BITS                      //
////                                    11 = 8 DATA BITS                      //
////                                                                          //
////    STB:  NUMBER OF STOP BITS  0 = 1 STOP BIT (DEFAULT)                   //
////                               1 = 1.5 STOP BITS (DATA LENGTH 5 BITS)     //
////                               1 = 2 STOP BITS   (DATA LENGTH 6,7,8 BITS) //
////                                                                          //
////    PEN:  PARITY ENABLE                                                   //
////    EPS:  EVEN PARITY SELECT                                              //
////    SP:   SET PARITY                                                      //
////                                  SP EPS PEN      PARITY SELECTION        //
////                                  X   X   0       NO PARITY               //
////                                  0   0   1       ODD PARITY              //
////                                  0   1   1       EVEN PARITY             //
////                                  1   0   1       forCE PARITY "1"        //
////                                  1   1   1       forCE PARITY "0"        //
////                                                                          //
////    SB:   SET BREAK When enable the Break control bit causes a break      //
////          condition to be transmitted (the TX output is forced to a logic //
////          0 state). This condition exits until disabled by resetting this //
////          bit to a logic 0.                                               //
////                                                                          //
////    DLAB: DIVIS| LATCH ACCESS BIT:   0 = Divis| latch disable (default) //
////                                      1 = Divis| latch enabled           //
////          ~e: Because we use ISP solution to reconfigure Baud Rate,     //
////                this bit is omitted as well as the Baud Rate Register.    //
////                                                                          //
////    =============================                                         //
////   | IIR (INTERRUPT ID REGISTER) |                                        //
////    ===================================================================   //
////   |   0   |   0   |   0   |   0   | INT 2 | INT 1 | INT 0 | INT STAT  |  //
////    ===================================================================   //
////                                                                          //
////   PRIOTITY LEVEL  BIT-3  BIT-2  BIT-1  BIT-0  SOURCE OF INTERRUPT        //
////   NONE              0      0      0      1    NONE                       //
////   HIGHEST           0      1      1      0    LSR (OE/PE/FE/BI)          //
////   2nd               0      1      0      0    RxRDY (Receiver Data Ready)//
////   3rd               0      0      1      0    THRE (THR Empty)           //
////   4th               0      0      0      0    MSR (Modem Status Register)//
////                                                                          //
////      In the 16450 Mode Bit-3 is 0.                                       //
////                                                                          //
////    =================================                                     //
////   | IER (INTERRUPT ENABLE REGISTER) |                                    //
////    ===============================================================       //
////   |   0   |   0   |   0   |   0   |  MSI  |  RLSI |  THRI | RHRI  |      //
////    ===============================================================       //
////                                                                          //
////   RBRI:    Receiver Buffer Register Interrupt (1 = Enable, 0 = Disble)   //
////   THRI:    Transmitter Hold Register Interrupt (1 = Enalbe, 0 = Disble)  //
////   RLSI:    Receiver Line Status Interrupt (1 = Enalble, 0 = Disble)      //
////   MSI:     Modem Status Interrupt (1 = Enable, 0 = Disble)               //
////                                                                          //
////    =============================                                         //
////   | MSR (MODEM STATUS REGISTER) |                                        //
////    ===============================================================       //
////   |  DCD  |  RI   |  DSR  |  CTS  |  DDCD |  TERI |  DDSR |  DCTS |      //
////    ===============================================================       //
////                                                                          //
////   DCD:       Data Carrier Detect                                         //
////   RI:        Ring Indicat|                                              //
////   DSR:       Data Set Ready                                              //
////   CTS:       Clear To Send                                               //
////   DDCD:      Delta Data Carrier Detect                                   //
////   TERI:      Trailing Edge Ring Indicat|                                //
////   DDSR:      Delta Data Set Ready                                        //
////   DCTS:      Delta Clear to Send                                         //
////   Bit0-3 are set to 1'b1 whenever a control input from the MODEM changes  //
////   state, & an Modem Stauts. Interrupt is generated. They are reset to  //
////   1'b0 whenever the CPU reads the Modem Status Register.                  //
////                                                                          //
////    ==============================                                        //
////   | MCR (MODEM CONTROL REGISTER) |                                       //
////    ===============================================================       //
////   |   0   |   0   |   0   | LOOP* | OUT2* | OUT1* |  RTS  |  DTR  |      //
////    ===============================================================       //
////                                                                          //
////   DTR:       Data Terminal Ready                                         //
////   RTS:       Request To Send                                             //
////   OUT1:      Auxiliary User-defined Output 1 (~ Implemented)           //
////   OUT2:      Auxiliary User-defined Output 2 (~ Implemented)           //
////   LOOP:      Local Loopback for diagnostic testing of the UART           //
////              (~ Implemented)                                           //
////                                                                          //
////   ~e:      OUT1, OUT2 & LOOP are ~ implemented                     //
////                                                                          //
////////////////////////////////////////////////////////////////////////////////




`timescale 1ns/1ps


`define C_S_AXI_DATA_WIDTH 32
`define C_S_AXI_ADDR_WIDTH 32
module axi_lite_slave (
     S_AXI_ACLK,
     S_AXI_ARESETN,
     S_AXI_AWADDR,
     S_AXI_AWVALID,
     S_AXI_AWREADY,
     S_AXI_WDATA,
     S_AXI_WSTRB,
     S_AXI_WVALID,
     S_AXI_WREADY,
     S_AXI_BRESP,
     S_AXI_BVALID,
     S_AXI_BREADY,
     S_AXI_ARADDR,
     S_AXI_ARVALID,
     S_AXI_ARREADY,
     S_AXI_RDATA,
     S_AXI_RRESP,
     S_AXI_RVALID,
     S_AXI_RREADY,
  
     Clk16X      , // UART internal clock

//Process| interface
     DDIS        , // Driver disable
     INTR        , // Interrupt
    // Registers
     RBR         , // Receiver Buffer Reg
     THR         , // Transmitter Holding Reg
     MSR         , // Modem Status Reg
     MCR         , // Modem Control Reg
    // Rising edge of registers read/write strobes
     RbrRDn_re   , // pulse indicating rising of RbrRDn_r
     ThrWRn_re   , // pulse indicating rising of ThrWRn_r
     LsrRDn_re   , // pulse indicating rising of LsrRDn_r
     MsrRDn_re   , // pulse indicating rising of MsrRDn_r
    // Receiver/Transmitter control
     DataBits    ,
     StopBits    ,
     ParityEnable,
     ParityEven  ,
     ParityStick ,
     TxBreak     ,
    // Receiver/Transmitter status
     RxRDY       ,
     OverrunErr  ,
     ParityErr   ,
     FrameErr    ,
     BreakInt    ,
     THRE        ,
     TEMT        , 
    // Clear LSR
     OE_re       ,
     PE_re       ,
     FE_re       ,
     BI_re 
 ); 
////////////////////////////////////////////////////////////////////
//

 input                                S_AXI_ACLK;
 input                                S_AXI_ARESETN;
 input   [`C_S_AXI_ADDR_WIDTH - 1:0]  S_AXI_AWADDR;
 input                                S_AXI_AWVALID;
 output                               S_AXI_AWREADY;
 input   [`C_S_AXI_DATA_WIDTH-1:0]    S_AXI_WDATA;
 input   [`C_S_AXI_DATA_WIDTH/8-1:0]  S_AXI_WSTRB;
 input                                S_AXI_WVALID;
 output                               S_AXI_WREADY;
 output  [1:0]                        S_AXI_BRESP;
 output                               S_AXI_BVALID;
 input                                S_AXI_BREADY;
 input   [`C_S_AXI_ADDR_WIDTH - 1:0]  S_AXI_ARADDR;
 input                                S_AXI_ARVALID;
 output                               S_AXI_ARREADY;
 output  [`C_S_AXI_DATA_WIDTH-1:0]    S_AXI_RDATA;
 output  [1:0]                        S_AXI_RRESP;
 output                               S_AXI_RVALID;
 input                                S_AXI_RREADY;

 input           Clk16X      ;
 output          DDIS        ; // Driver disable
 output          INTR        ; // Interrupt
        // Registers
 input  [7:0]    RBR         ; // Receiver Buffer Reg
 output [7:0]    THR         ; // Transmitter Holding Reg
 input  [7:0]    MSR         ; // Modem Status Reg
 output [1:0]    MCR         ; // Modem Control Reg
        // Rising edge of registers read/write strobes
 output          RbrRDn_re   ; // pulse indicating rising of RbrRDn_r
 output          ThrWRn_re   ; // pulse indicating rising of ThrWRn_r
 output          LsrRDn_re   ; // pulse indicating rising of LsrRDn_r
 output           MsrRDn_re   ; // pulse indicating rising of MsrRDn_r
         // Receiver/Transmitter control
 output [1:0]    DataBits    ;
 output [1:0]    StopBits    ;
 output          ParityEnable;
 output          ParityEven  ;
 output          ParityStick ;
 output          TxBreak     ;
         // Receiver/Transmitter status
 input           RxRDY       ;
 input           OverrunErr  ;
 input           ParityErr   ;
 input           FrameErr    ;
 input           BreakInt    ;
 input           THRE        ;
 input           TEMT        ;

 output         OE_re        ;
 output         PE_re        ;
 output         FE_re        ;
 output         BI_re        ;

/////////////////////////////////////////////////////////////////
 wire                                 S_AXI_ACLK;
 wire                                 S_AXI_ARESETN;
 wire    [`C_S_AXI_ADDR_WIDTH - 1:0]  S_AXI_AWADDR;
 wire                                 S_AXI_AWVALID;
 wire                                 S_AXI_AWREADY;
 wire    [`C_S_AXI_DATA_WIDTH-1:0]    S_AXI_WDATA;
 wire    [`C_S_AXI_DATA_WIDTH/8-1:0]  S_AXI_WSTRB;
 wire                                 S_AXI_WVALID;
 wire                                 S_AXI_WREADY;
 wire    [1:0]                        S_AXI_BRESP;
 wire                                 S_AXI_BVALID;
 wire                                 S_AXI_BREADY;
 wire    [`C_S_AXI_ADDR_WIDTH - 1:0]  S_AXI_ARADDR;
 wire                                 S_AXI_ARVALID;
 wire                                 S_AXI_ARREADY;
 wire    [`C_S_AXI_DATA_WIDTH-1:0]    S_AXI_RDATA;
 wire    [1:0]                        S_AXI_RRESP;
 wire                                 S_AXI_RVALID;
 wire                                 S_AXI_RREADY;

//----------UART internal signals-------------//

 reg     [7:0]      THR       ; // Transmitter Holding Reg
 reg     [1:0]      MCR       ; // Modem Control Reg

 wire    [6:0]     LSR       ;
 reg     [6:0]     LCR       ;
 wire    [3:0]     IIR       ;
 reg     [3:0]     IER       ;

 reg               ThrWRn_re  ; // THR write 
//reg              RbrRDn_re  ; // RBR read 
 reg               LsrRDn_re  ; // LSR read 
 reg               MsrRDn_re  ; // MSR read 
 reg               IirRDn_re  ; // Rising edge of IIR read strobe

 wire              RxRDY_Int   ;
 wire              THRE_Int    ;
 wire              DataErr_Int ;
 wire              Modem_Int   ;

 wire              DataErr     ;
 wire              ModemStat   ;

 reg               OE_re     ;   //Clear OverrunErr
 reg               PE_re     ;   //Clear ParityErr
 reg               FE_re     ;   //Clear FrameErr
 reg               BI_re     ;   //Clear BreakInt


////////////////////////////////////////////////////////////////////////////
// local parameter for addressing 32 bit / 64 bit C_S_AXI_DATA_WIDTH
// ADDR_LSB is used for addressing 32/64 bit registers/memories
// ADDR_LSB = 2 for 32 bits (n downto 2)
// ADDR_LSB = 3 for 64 bits (n downto 3)

////////////////////////////////////////////////////////////////////////////
// function called clogb2 that returns an integer which has the
// value of the ceiling of the log base 2.
 function integer clogb2 (input integer bd);
 integer bit_depth;
 begin
   bit_depth = bd;
   for(clogb2=0; bit_depth>0; clogb2=clogb2+1)
     bit_depth = bit_depth >> 1;
   end
 endfunction

 localparam integer ADDR_LSB = clogb2(`C_S_AXI_DATA_WIDTH/8)-1;
 localparam integer ADDR_MSB = `C_S_AXI_ADDR_WIDTH;


 reg [1 :0]                     axi_rresp;
 reg [1 :0]                     axi_bresp;
 reg                            axi_awready;
 reg                            axi_wready;
 reg                            axi_bvalid;
 reg                            axi_rvalid;
 reg [ADDR_MSB-1:0]             axi_awaddr;
 reg [ADDR_MSB-1:0]             axi_araddr;
 reg [`C_S_AXI_DATA_WIDTH-1:0]  axi_rdata;
 reg                            axi_arready;
 reg                            axi_arready_pre;
 
 wire                           slv_reg_rden;
 wire                           slv_reg_wren;
 reg [`C_S_AXI_DATA_WIDTH-1:0]  reg_data_out;
 integer                        byte_index;


// State Machine Definition
//type state_typ is (idle, int0, int1, int2, int3);
//wire Int_State : state_typ;
  
 reg [2:0] Int_State;
 parameter 
 idle = 3'b000,
 int0 = 3'b001,
 int1 = 3'b010,
 int2 = 3'b011,
 int3 = 3'b100; 

// UART Registers Address Map
 parameter A_RBR = 3'b000;
 parameter A_THR = 3'b000;
 parameter A_IER = 3'b001;
 parameter A_IIR = 3'b010;
 parameter A_LCR = 3'b011;
 parameter A_MCR = 3'b100;
 parameter A_LSR = 3'b101;
 parameter A_MSR = 3'b110;

////////////////////////////////////////////////////
 assign S_AXI_AWREADY = axi_awready;
 assign S_AXI_WREADY  = axi_wready;
 assign S_AXI_BRESP  = axi_bresp;
 assign S_AXI_BVALID = axi_bvalid;
 assign S_AXI_ARREADY = axi_arready;
 assign S_AXI_RDATA  = axi_rdata;
 assign S_AXI_RVALID = axi_rvalid;
 assign S_AXI_RRESP  = axi_rresp;

  always @( posedge S_AXI_ACLK )
  begin
    if ( S_AXI_ARESETN == 1'b0 )
      begin
        axi_awready <= 1'b0;
      end
    else
      begin
        if (~axi_awready && S_AXI_AWVALID && S_AXI_WVALID)
          begin
            axi_awready <= 1'b1;
          end
        else
          begin
            axi_awready <= 1'b0;
          end
      end
  end

  always @( posedge S_AXI_ACLK )
  begin
    if ( S_AXI_ARESETN == 1'b0 )
      begin
        axi_awaddr <= 0;
      end
    else
      begin
        if (~axi_awready && S_AXI_AWVALID && S_AXI_WVALID)
          begin
            axi_awaddr <= S_AXI_AWADDR;
          end
      end
  end

  always @( posedge S_AXI_ACLK )
  begin
    if ( S_AXI_ARESETN == 1'b0 )
      begin
        axi_wready <= 1'b0;
      end
    else
      begin
        if (~axi_wready && S_AXI_WVALID && S_AXI_AWVALID)
          begin
     
            axi_wready <= 1'b1;
          end
        else
          begin
            axi_wready <= 1'b0;
          end
      end
  end

  assign slv_reg_wren = axi_wready && S_AXI_WVALID && axi_awready && S_AXI_AWVALID;



//------------ Register Write--------------//

  always @( posedge S_AXI_ACLK )
  begin
    if ( S_AXI_ARESETN == 1'b0 )
      begin
        THR <= 'b0;
        IER <= 'b0;
        LCR <= 8'b00000011;
        MCR <= 'b0;     
      end
    else begin
      if (slv_reg_wren)
        begin
          case ( axi_awaddr[ADDR_MSB-1:ADDR_LSB] )
            A_THR :  THR <= S_AXI_WDATA[7 : 0];
            A_IER :  IER <= S_AXI_WDATA[3:0];
            A_LCR :  LCR <= {1'b0,S_AXI_WDATA[6:0]}; //8'b00000011; //S_AXI_WDATA[6:0];
            A_MCR :  MCR <= S_AXI_WDATA[1:0];
            default : begin
                     THR <= THR;
                     IER <= IER;
                     LCR <= LCR;
                     MCR <= MCR;
                     end
          endcase
        end
    end
  end

  always @( posedge S_AXI_ACLK )
  begin
    if ( S_AXI_ARESETN == 1'b0 )
      begin
        axi_bvalid  <= 0;
        axi_bresp   <= 2'b0;
      end
    else
      begin
        if (axi_awready && S_AXI_AWVALID && ~axi_bvalid && axi_wready && S_AXI_WVALID)
          begin
            // indicates a valid write response is available
            axi_bvalid <= 1'b1;
            axi_bresp  <= 2'b0; // 'OKAY' response
          end                   // work error responses in future
        else
          begin
            if (S_AXI_BREADY && axi_bvalid)
    
              begin
                axi_bvalid <= 1'b0;
              end
          end
      end
  end

  always @( posedge S_AXI_ACLK )
  begin
    if ( S_AXI_ARESETN == 1'b0 )
      begin
        axi_arready_pre <= 1'b0;
        axi_araddr  <= {ADDR_MSB{1'b0}};
      end
    else
      begin
        if (~axi_arready_pre && ~axi_arready && S_AXI_ARVALID)
          begin
            // indicates that the slave has acceped the valid read address
            axi_arready_pre <= 1'b1;
            axi_araddr  <= S_AXI_ARADDR;
          end
        else
          begin
            axi_arready_pre <= 1'b0;
          end
      end
  end

always @( posedge S_AXI_ACLK )
  begin
    if ( S_AXI_ARESETN == 1'b0 )
      begin
        axi_arready <= 1'b0;
      end
    else
      begin
        if (axi_arready_pre )
          begin
            axi_arready <= 1'b1;
          end
        else
          begin
            axi_arready <= 1'b0;
          end
      end
  end


  always @( posedge S_AXI_ACLK )
  begin
    if ( S_AXI_ARESETN == 1'b0 )
      begin
        axi_rvalid <= 0;
        axi_rresp  <= 0;
      end
    else
      begin
        if (axi_arready && S_AXI_ARVALID && ~axi_rvalid)
          begin
            // Valid read data is available at the read data bus
            axi_rvalid <= 1'b1;
            axi_rresp  <= 2'b0; // 'OKAY' response
          end
        else if (axi_rvalid && S_AXI_RREADY)
          begin
            // Read data is accepted by the master
            axi_rvalid <= 1'b0;
          end
      end
  end

  assign slv_reg_rden = axi_arready & S_AXI_ARVALID & ~axi_rvalid;

//------- Register Read-------------//

  always @( RBR, IIR, LSR, MSR, S_AXI_ARESETN, slv_reg_rden, axi_araddr)
  begin
    if ( S_AXI_ARESETN == 1'b0 )
      begin
        reg_data_out <= {`C_S_AXI_DATA_WIDTH{1'b0}};
      end
    else
      begin
        // Read address mux
        case ( axi_araddr[ADDR_MSB-1:ADDR_LSB] )
          A_RBR :   reg_data_out <= {{(`C_S_AXI_DATA_WIDTH-8){1'b0}}, RBR};
          A_IIR :   reg_data_out <= {{(`C_S_AXI_DATA_WIDTH-4){1'b0}}, IIR};
          A_LSR :   reg_data_out <= {{(`C_S_AXI_DATA_WIDTH-1){1'b0}}, LSR};
          A_MSR :   reg_data_out <= {{(`C_S_AXI_DATA_WIDTH-8){1'b0}}, MSR};
          default:  reg_data_out <= {`C_S_AXI_DATA_WIDTH{1'b0}};
        endcase
        //end
      end
  end

  always @( posedge S_AXI_ACLK )
  begin
    if ( S_AXI_ARESETN == 1'b0 )
      begin
        axi_rdata  <= 0;
      end
    else
      begin

        if (axi_arready && S_AXI_ARVALID && ~axi_rvalid)
          begin
            axi_rdata <= reg_data_out;     // register read data
          end
      end
  end


////////////////////////////////////////////////////////////////////////////////
// Registers Read/Write Control wires
////////////////////////////////////////////////////////////////////////////////

always @(posedge S_AXI_ACLK)
  begin 
    if(S_AXI_ARESETN == 1'b0)
      ThrWRn_re <=  1'b0;
    else if (slv_reg_wren&&(axi_awaddr[ADDR_MSB-1:ADDR_LSB]== A_THR ))
      ThrWRn_re <=  1'b1;
    else 
      ThrWRn_re <=  1'b0;
  end

assign RbrRDn_re = axi_arready_pre & S_AXI_ARVALID & ~axi_rvalid &&(axi_araddr[ADDR_MSB-1:ADDR_LSB]== A_RBR );

always @(posedge S_AXI_ACLK)
  begin 
    if(S_AXI_ARESETN == 1'b0)
     begin
      LsrRDn_re <= 1'b0;
     end
    else if (slv_reg_rden&&(axi_araddr[ADDR_MSB-1:ADDR_LSB]== A_LSR ))
      begin
        LsrRDn_re <=  1'b1;
      end
    else 
     begin
      LsrRDn_re <= 1'b0;
     end
  end

//Clear  BI   |  FE   |  PE   |  OE 
always @(posedge S_AXI_ACLK)
  begin 
    if(S_AXI_ARESETN == 1'b0)
     begin
      OE_re     <= 1'b0;
      PE_re     <= 1'b0;
      FE_re     <= 1'b0;
      BI_re     <= 1'b0;
     end
    else if (slv_reg_wren&&(axi_awaddr[ADDR_MSB-1:ADDR_LSB]== A_LSR ))
      begin
        if(S_AXI_WDATA[1])     
          OE_re   <= 1'b1;
        if( S_AXI_WDATA[2])
          PE_re   <= 1'b1;
        if (S_AXI_WDATA[3])
          FE_re   <= 1'b1;
        if (S_AXI_WDATA[4])
          BI_re   <= 1'b1;
      end
   else 
     begin
      OE_re     <= 1'b0;
      PE_re     <= 1'b0;
      FE_re     <= 1'b0;
      BI_re     <= 1'b0;
     end
  end


always @(posedge S_AXI_ACLK)
  begin 
    if(S_AXI_ARESETN == 1'b0)
      IirRDn_re <=  1'b0;
    else if (slv_reg_rden&&(axi_araddr[ADDR_MSB-1:ADDR_LSB]== A_IIR ))
      IirRDn_re <=  1'b1;
    else 
      IirRDn_re <=  1'b0;
  end

always @(posedge S_AXI_ACLK)
  begin 
    if(S_AXI_ARESETN == 1'b0)
      MsrRDn_re <=  1'b0;
    else if (slv_reg_rden&&(axi_araddr[ADDR_MSB-1:ADDR_LSB]== A_MSR ))
      MsrRDn_re <=  1'b1;
    else 
      MsrRDn_re <=  1'b0;
  end



////////////////////////////////////////////////////////////////////////////////
//  Line Control Register
////////////////////////////////////////////////////////////////////////////////

  // Databits : "00"=5-bit, "01"=6-bit, "10"=7-bit, "11"=8-bit
  assign DataBits = LCR[1:0];

  // Stopbits : "00"=1-bit, "01"=1.5-bit(5-bit data), "10"=2-bit(6,7,8-bit data)
  assign StopBits = (LCR[2]==1'b0) ? 2'b00 :
                    ((LCR[2]==1'b1) & (LCR[1:0]==2'b00)) ? 2'b01 :
                    2'b10;
  // ParityEnable : 1'b0=Parity Bit Enable, 1'b1=Parity Bit Disable
  assign ParityEnable = LCR[3];

  // ParityEven : 1'b0=Even Parity Selected, 1'b1=Odd Parity Selected
  assign ParityEven = LCR[4];

  // ParityStick : 1'b0=Stick Parity Disable, 1'b1=Stick Parity Enable
  assign ParityStick = LCR[5];

  // TxBreak : 1'b0=Disable BREAK assertion, 1'b1=Assert BREAK
  assign TxBreak = LCR[6];

////////////////////////////////////////////////////////////////////////////////
//  Line Status Register
////////////////////////////////////////////////////////////////////////////////

  assign LSR = {TEMT, THRE, BreakInt, FrameErr, ParityErr, OverrunErr, RxRDY};

////////////////////////////////////////////////////////////////////////////////
// Interrupt Arbitrat|
////////////////////////////////////////////////////////////////////////////////

  // Int is the common interrupt line for all internal UART events
  assign INTR = RxRDY_Int | THRE_Int | DataErr_Int | Modem_Int;

  // Receiving Data Err| Flags including Overrun, Parity, Framing & Break
  assign DataErr = OverrunErr | ParityErr | FrameErr | BreakInt;

  // Whenever bit0,1,2,|3 is set to 1'b1, a Modem Status Interrupt is generated
  assign ModemStat = MSR[0] | MSR[1] | MSR[2] | MSR[3];

//  Int_Arbit_Proc: 
always @(posedge S_AXI_ACLK)
  begin
    if (S_AXI_ARESETN == 1'b0) 
      Int_State <= idle;

    else
      case (Int_State)
        idle :

          if ((IER[2]==1'b1) & (DataErr==1'b1))
            Int_State <= int0;
          else if ((IER[0]==1'b1) & (RxRDY==1'b1))
            Int_State <= int1;
          else if ((IER[1]==1'b1) & (THRE==1'b1))
            Int_State <= int2;
          else if ((IER[3]==1'b1) & (ModemStat==1'b1))
            Int_State <= int3;

        int0 :
          if (LsrRDn_re==1'b1) 
            // Clear Receiver Line Status Interrupt after LSR read
            Int_State <= idle;

        int1 :
          if (RxRDY==1'b0)
            // Clear Received Data Available Interrupt after RBR read(RxRDY=0)
            Int_State <= idle;

        int2 :
          if ((IirRDn_re==1'b1) | (THRE==1'b0))
            // Clear THR Empty Interrupt after IIR read | THR write(THRE=0)
            Int_State <= idle;

        int3 :
          if (MsrRDn_re==1'b1)
            // Clear MODEM Status Interrupt after MSR read
            Int_State <= idle;

        default :
          Int_State <= idle;
        endcase
   end

  // Set Receiver Line Status Interrupt
  assign DataErr_Int = (Int_State==int0) ? 1'b1 : 1'b0;

  // Set Received Data Available Interrupt
  assign RxRDY_Int = (Int_State==int1) ? 1'b1 : 1'b0;

  // Set THR Empty Interrupt
  assign THRE_Int = (Int_State==int2) ? 1'b1 : 1'b0;

  // Set MODEM Status Interrupt
  assign Modem_Int = (Int_State==int3) ? 1'b1 : 1'b0;

  // Update IIR
  assign IIR = (Int_State==int0) ? 4'b1110 : // 1st Pri|ity Interrupt
               (Int_State==int1) ? 4'b1100 : // 2nd Pri|ity Interrupt
               (Int_State==int2) ? 4'b1010 : // 3rd Pri|ity Interrupt
               (Int_State==int3) ? 4'b1000 : // 4th Pri|ity Interrupt
                                   4'b1001 ; // No Interrupt Pending


  // Driver Disable goes low whenever the CPU is reading data from the UART
  assign DDIS = ~slv_reg_rden ;

endmodule



