// ===========Oooo==========================================Oooo========
// =  Copyright (C) 2014 Jinan Gowin Semiconductor Technology Co.,Ltd.
// =                     All rights reserved.
// =====================================================================
//
//  __      __      __
//  \ \    /  \    / /   [File name   ] rxcver.v
//   \ \  / /\ \  / /    [Description ] Verilog file for the UART Receiver Module
//    \ \/ /  \ \/ /     [Timestamp   ] Tue Aug 27 15:30:00 2015
//     \  /    \  /      [version     ] 1.0
//      \/      \/       
// --------------------------------------------------------------------
// Code Revision History :
// --------------------------------------------------------------------
// Ver: | Author |Mod. Date |Changes Made:
// V1.0 | XX     |27/08/15  |Initial version
// ===========Oooo==========================================Oooo========
//
//    <Global reset and clock>
//      Reset       : Master reset
//      Clk16X      : UART internal clock
//
//    <Register>
//      RBR         : Receiver Buffer Register
//
//    <Rising edge of RBR, LSR read strobes>
//      RbrRDn_re   : one Clk16X width pulse indicating rising edge of RbrRDn_r
//      LsrRDn_re   : one Clk16X width pulse indicating rising edge of LsrRDn_r
//
//    <Receiver input>
//      SIN         : Receiver serial input
//
//    <Receiver control>
//      Databits    : "00"=5-bit, "01"=6-bit, "10"=7-bit, "11"=8-bit
//      ParityEnable: 1'b0=Parity Bit Enable, 1'b1=Parity Bit Disable
//      ParityEven  : 1'b0=Even Parity Selected, 1'b1=Odd Parity Selected
//      ParityStick : 1'b0=Stick Parity Disable, 1'b1=Stick Parity Enable
//
//    <Receiver/Transmitter status>
//      RxRDY       : RBR data is ready to be read
//      OverrunErr  : Overrun error
//      ParityErr   : Parity error
//      FrameErr    : Frame error
//      BreakInt    : BREAK interrupt
//
// ////////////////////////////////////////////////////////////////////


module Rxcver
   (
    // Global reset and clock
    S_AXI_ACLK  ,
    Reset       , // Master reset
    Clk16X      , // UART internal clock
    // Register
    RBR         , // Receiver Buffer Reg
    // Rising edge of RBR, LSR read strobes
    RbrRDn_re   , // pulse indicating rising of RbrRDn_r
    LsrRDn_re   , // pulse indicating rising of LsrRDn_r
    // Receiver input
    SIN         ,
    // Receiver control
    DataBits    , // Data bits length
    ParityEnable, // 0= Parity Disabled; 1= Parity Enabled
    ParityEven  , // 0= Odd Parity; 1= Even Parity
    ParityStick , // 0= Stick Disabled; 1= Stick Enabled
    // Receiver status
    RxRDY       , // Receiver data ready to read
    OverrunErr  , // Receiver overrun error flag
    ParityErr   , // Receiver parity error flag
    FrameErr    , // Receiver framing error flag
    BreakInt    , // Receiver BREAK interrupt flag
    // Clear LSR
     OE_re       ,
     PE_re       ,
     FE_re       ,
     BI_re 

  );

input           S_AXI_ACLK  ;
input           Reset       ; // Master reset
input           Clk16X      ; // UART internal clock
    // Register
output [7:0]    RBR         ; // Receiver Buffer Reg
    // Rising edge of RBR, LSR read strobes
input           RbrRDn_re   ; // pulse indicating rising of RbrRDn_r
input           LsrRDn_re   ; // pulse indicating rising of LsrRDn_r
    // Receiver input
input           SIN         ;
    // Receiver control
input [1:0]     DataBits    ; // Data bits length
input           ParityEnable; // 0= Parity Disabled; 1= Parity Enabled
input           ParityEven  ; // 0= Odd Parity; 1= Even Parity
input           ParityStick ; // 0= Stick Disabled; 1= Stick Enabled
    // Receiver status
output          RxRDY       ; // Receiver data ready to read
output          OverrunErr  ; // Receiver overrun error flag
output          ParityErr   ; // Receiver parity error flag
output          FrameErr    ; // Receiver framing error flag
output          BreakInt    ; // Receiver BREAK interrupt flag
   // Clear LSR
input         OE_re        ;
input         PE_re        ;
input         FE_re        ;
input         BI_re        ;


  reg [3:0]   NumDataBitReceived_r ;

  reg [7:0]   RSR         ;
  reg         RxPrtyErr   ;
  reg         RxFrmErr    ;
  reg         RxIdle_r    ;
  reg         RbrDataRDY  ;
  reg [3:0]   CNT_r       ;

  reg         Hunt_r      ;
  reg         HuntOne_r   ;

  reg         SIN1_r      ;
  reg         RxFrmErr1_r ;
  reg         RxIdle1_r   ;

  reg         OverrunErr_r;
  reg         ParityErr_r ;
  reg         FrameErr_r  ;
  reg         BreakInt_r  ;

  reg         SampledOnce ;

  // Receiver Clock Enable Signal
  reg         RxClkEn     ;

  reg [7:0]   RBR_r       ;

  wire        full;
  wire        empty;
  reg         RxIdle_r_1,RxIdle_r_2;
  wire        rise_edge_RxIdle_r;
  
  // State Machine Definition
  //type state_typ is (idle, shift, parity, stop);
  //signal Rx_State : state_typ;

  parameter
  idle=2'b00,
  shift=2'b01,
  parity=2'b10,
  stop=2'b11;
  
  reg [1:0] Rx_State;

  // Attributes for ispMACH5000VG to get higher performance
  //   These can be removed when the UART design is targeted to other devices.
  //ATTRIBUTE SYN_KEEP : integer;
  //ATTRIBUTE SYN_KEEP OF RxPrtyErr, NumDataBitReceived_r, RSR : SIGNAL IS 1;
  //ATTRIBUTE SYN_KEEP OF Hunt_r, HuntOne_r, ParityErr_r, FrameErr_r : SIGNAL IS 1;
  //ATTRIBUTE SYN_KEEP OF BreakInt_r, RBR_r, OverrunErr_r, RbrDataRDY : SIGNAL IS 1;
  //ATTRIBUTE SYN_KEEP OF RxFrmErr, RxIdle_r : SIGNAL IS 1;
  //ATTRIBUTE OPT : string;
  //ATTRIBUTE OPT OF RxPrtyErr, NumDataBitReceived_r, RSR : SIGNAL IS "KEEP";
  //ATTRIBUTE OPT OF Hunt_r, HuntOne_r, ParityErr_r, FrameErr_r : SIGNAL IS "KEEP";
  //ATTRIBUTE OPT OF BreakInt_r, RBR_r, OverrunErr_r, RbrDataRDY : SIGNAL IS "KEEP";
  //ATTRIBUTE OPT OF RxFrmErr, RxIdle_r : SIGNAL IS "KEEP";


////////////////////////////////////////////////////////////////////////////////
// Generate RxClkEn signal
////////////////////////////////////////////////////////////////////////////////

 always @(posedge S_AXI_ACLK) 
          if(Reset==0) 
          begin
             RxIdle_r_1 <= 1;
             RxIdle_r_2 <= 1;
          end
          else begin
                 RxIdle_r_1 <=  RxIdle_r;
                 RxIdle_r_2  <= RxIdle_r_1;
          end
 assign   rise_edge_RxIdle_r = RxIdle_r_1&(~RxIdle_r_2);




  // RxClkEn : serial port data receiving clock enable
//  RxCLK_Proc: process 
always @(negedge Reset, posedge S_AXI_ACLK)
  begin
    if (Reset == 0) 
      RxClkEn <= 1'b0;
    else
      if (CNT_r==4'b0110)
        RxClkEn <= 1'b1;
      else
        RxClkEn <= 1'b0;
  end

  // CNT_r : 4-bit counter for RxClkEn waveform generation
  //CNT_Proc: process 
  always @(negedge Reset, posedge Clk16X)
  begin
    if (Reset == 0)
      CNT_r <= 'b0;
    else
      if ((Rx_State != idle) | (Hunt_r))
        // Increment count when not idle or when Hunt_r is TRUE
        CNT_r <= CNT_r + 1;
      else if (SampledOnce==1'b1)
        // Adjust 2 clks forward for RxClkEn during the resync after framing error
        CNT_r <= 'b0010;
      else
        CNT_r <= 'b0;
  end

////////////////////////////////////////////////////////////////////////////////
// Generate Hunt_r
////////////////////////////////////////////////////////////////////////////////

  // Hunt_r : will be TRUE when start bit is found
  //Hunt_r_Proc: process 
  always @(negedge Reset, posedge Clk16X)
  begin
    if (Reset == 0) 
      Hunt_r <= 1'b0;
    else begin
      if ((Rx_State==idle) & (SIN==1'b0) & (SIN1_r==1'b1))
        // Set Hunt_r when SIN falling edge is found at the idle state
        Hunt_r <= 1'b1;
      else if ((SampledOnce==1'b1) & (SIN==1'b0))
        // Start bit is successfully sampled twice after framing error
        // set Hunt_r "true" for resynchronizing of next frame
        Hunt_r <= 1'b1;
      else if ((RxIdle_r==1'b0) | (SIN==1'b1))
        // Clear Hunt_r when data shifting starts or when SIN returns to 1'b1
        Hunt_r <= 1'b0;
      end
  end

  // HuntOne_r :
  //   HuntOne_r, used for BI flag generation, indicates that there is at
  //   least a 1'b1 in the (data + parity + stop) bits of the frame.
  //   Break Interrupt flag(BI) is set to 1'b1 whenever the received input
  //   is held at the 1'b0 state for all bits in the frame (Start bit +
  //   Data bits + Parity bit + Stop bit).  So, as long as HuntOne_r is still
  //   low after all bits are received, BI will be set to 1'b1.
  //HuntOne_r_Proc: process
  always @(posedge S_AXI_ACLK, negedge Reset)
  begin
    if (Reset == 0)
      HuntOne_r <= 1'b0;
    else begin
      if (Hunt_r)
        HuntOne_r <= 1'b0;
      else if ((RxIdle_r==1'b0) & (CNT_r[3]==1'b1) & (SIN==1'b1))
        HuntOne_r <= 1'b1;
      end
  end

  // RbrDataRDY :
  //   This will be set to indicate that the data in RBR is ready for read and
  //   will be cleared after RBR is read.
  //RbrDataRDY_Proc: process
  always @(posedge S_AXI_ACLK, negedge Reset)
  begin
    if (Reset == 0) 
      RbrDataRDY <= 1'b0;
    else begin
      if (!empty)
        // set RbrDataRDY at RxIdle_r rising edge
        RbrDataRDY <= 1'b1;
      else if (empty)
        // clear RbrDataRDY when RBR is read by CPU
        RbrDataRDY <= 1'b0;
      end 
  end

  // SampledOnce :
  //   This will be set for one Clk16X clock after a framing error occurs not
  //   because of BREAK and a low SIN signal is sampled by the Clk16X right
  //   after the sample time of the Stop bit which causes the framing error.
  //SampledOnce_Proc: process 
  always @(*)
  begin
    if (Reset == 0) 
      SampledOnce <= 1'b0;
    else
      if ((RxFrmErr==1'b1) & (RxFrmErr1_r==1'b0) &
         (SIN==1'b0) & (HuntOne_r==1'b1))
        // Start bit got sampled once
        SampledOnce <= 1'b1;
      else
        SampledOnce <= 1'b0;
  end

  // RxIdle_r Flag
  //RxIdle_Proc: process
  always @(negedge Reset, posedge S_AXI_ACLK )
  begin
    if (Reset == 0) 
      RxIdle_r <= 1'b1;
    else
      if (Rx_State==idle)
        RxIdle_r <= 1'b1;
      else if (CNT_r[3]==1'b1)
        RxIdle_r <= 1'b0;
  end

////////////////////////////////////////////////////////////////////////////////
// Receiver Finite State Machine
////////////////////////////////////////////////////////////////////////////////

  //Shift_data_Proc: process
  always @(posedge Clk16X, negedge Reset)
  begin
    if (Reset == 0) begin
      RSR <= 'b0;
      NumDataBitReceived_r <= 'b0;
      RxPrtyErr <= 1'b1;
      RxFrmErr <= 1'b0;
      Rx_State <= idle;
      end
    else
      case (Rx_State)
        idle :
          if ((RxIdle_r==1'b1) & (SIN==1'b0) & (RxClkEn==1'b1)) begin
            RSR <= 'b0;
            NumDataBitReceived_r <= 'b0;
            //  RxPrtyErr:
            //    RxPrtyErr is a dynamic Parity Error indicator which is
            //    initialized to 0 for even parity and 1 for odd parity.
            //    For odd parity, if there are odd number of 1'b1s in the
            //    (data + parity) bits, the XOR will bring RxPrtyErr back to 0
            //    which means no parity error, otherwise RxPrtyErr will be 1 to
            //    indicate a parity error.
            RxPrtyErr <= ~ParityEven;
            RxFrmErr <= 1'b0;
            Rx_State <= shift;
            end 
        shift :
          if (RxClkEn==1'b1) begin
            RSR <= {SIN,RSR[7:1]};
            RxPrtyErr <= RxPrtyErr ^ SIN; // Update RxPrtyErr dynamically
            NumDataBitReceived_r <= NumDataBitReceived_r + 1;
            if ((DataBits==2'b00 & NumDataBitReceived_r==4) |
                (DataBits==2'b01 & NumDataBitReceived_r==5) |
                (DataBits==2'b10 & NumDataBitReceived_r==6) |
                (DataBits==2'b11 & NumDataBitReceived_r==7))
              if (ParityEnable==1'b0)
                Rx_State <= stop;
              else
                Rx_State <= parity;
          end
        parity :
          if (RxClkEn==1'b1) begin
            if (ParityStick==1'b0) begin
              RxPrtyErr <= RxPrtyErr ^ SIN; // Update RxPrtyErr dynamically
              Rx_State  <= stop;
            end
            else begin
              // ParityStick=1'b1 means Stick Parity is enabled.  In this case,
              // the accumulated dynamic RxPrtyErr result will be ignored.  A new
              // value will be assigned to RxPrtyErr based on the even/odd parity
              // mode setting and the SIN sampled in parity bit.
              //    ParityEven=1'b0(odd parity):
              //       SIN needs to be 1'b1, otherwise it's a stick parity error.
              //    ParityEven=1'b1(even parity):
              //       SIN needs to be 1'b0, otherwise it's a stick parity error.
              if (ParityEven==1'b0) begin
                RxPrtyErr <= ~SIN;
                Rx_State  <= stop;
              end
              else begin
                RxPrtyErr <= SIN;
                Rx_State <= stop;
              end 
             end
          end
        stop :
          if (RxClkEn==1'b1) begin
            // The Receiver checks the 1st Stopbit only regardless of the number
            // of Stop bits selected.
            // Stop bit needs to be 1'b1, otherwise it's a Framing error
            RxFrmErr <= ~SIN;
            Rx_State <= idle;
          end
        default :
          if (RxClkEn==1'b1)
            Rx_State <= idle;
      endcase
  end

////////////////////////////////////////////////////////////////////////////////
// Receiver Buffer Register
////////////////////////////////////////////////////////////////////////////////

  //RBR_Proc: process
//  always @(posedge Clk16X, negedge Reset)
  always @(*)
  begin
//    if (Reset == 0) 
//      RBR_r <= 'b0;
//    else 
//      if ((RxIdle_r==1'b1) & (RxIdle1_r==1'b0))
        // Update RBR at RxIdle_r ris|ing edge
        case (DataBits)
          2'b00 :   // 5-bit data
            RBR_r <= {3'b000,RSR[7:3]};
          2'b01 :   // 6-bit data
            RBR_r <=  {2'b00,RSR[7:2]};
          2'b10 :   // 7-bit data
            RBR_r <=   {1'b0,RSR[7:1]};
          default : // 8-bit data
            RBR_r <= RSR;
        endcase
  end


fifo_syn Rx_FIFO (.datain(RBR_r), .rd(RbrRDn_re) , .wr(rise_edge_RxIdle_r), .rst(Reset), .clk(S_AXI_ACLK), .dataout(RBR), .full(full), .empty(empty));


////////////////////////////////////////////////////////////////////////////////
// Delayed Signals for edge detections
////////////////////////////////////////////////////////////////////////////////
  //Delay_Signals_Proc: process
  always @(posedge Clk16X, negedge Reset)
  begin
    if (Reset == 0) begin
      SIN1_r <= 1'b0;
      RxFrmErr1_r <= 1'b1;
      end
    else begin
      // SIN1_r : Signal for falling edge detection of signal SIN
      SIN1_r <= SIN;
      // RxFrmErr1_r :
      //   a delayed version of RxFrmErr for detacting the rising edge
      //   used to resynchronize the next frame after framing error
      RxFrmErr1_r <= RxFrmErr;
      end
   end

  always @(posedge S_AXI_ACLK, negedge Reset)
  begin
    if (Reset == 0) begin
      RxIdle1_r <= 1'b1;
      end
    else begin
      // RxIdle1_r : Signal for rising edge detection of signal RxIdle_r
      RxIdle1_r <= RxIdle_r;
      end
   end

////////////////////////////////////////////////////////////////////////////////
// Generate Error Flags
////////////////////////////////////////////////////////////////////////////////

  // Receiver Error Flags in LSR
  //   OverrunErr(OE), ParityErr_r(PE), FrameErr_r(FE), BreakInt_r(BI)
  //   will be set to reflect the SIN line status only after the whole frame
  //   (Start bit + Data bits + Parity bit + Stop bit) is received.  A rising
  //   edge of RxIdle_r indicates the whole frame is received.
  //Error_Flags_Proc: process
  always @(posedge S_AXI_ACLK, negedge Reset)
  begin
    if (Reset == 0) begin
      ParityErr_r <= 1'b0;
      FrameErr_r <= 1'b0;
      BreakInt_r <= 1'b0;
      end
    else
     begin
      if ((RxIdle_r==1'b1) & (RxIdle1_r==1'b0)) begin // update at RxIdle_r rising
        // Set ParityErr_r flag if RxPrtyErr is 1 when Parity enable
        ParityErr_r <= (ParityErr_r | RxPrtyErr) & ParityEnable;
        // Set FrameErr_r flag if RxFrmErr is 1(Stop bit is sampled low)
        FrameErr_r <= FrameErr_r | RxFrmErr;
        // Set BreakInt_r flag if HuntOne_r is still low
        BreakInt_r <= BreakInt_r | (~HuntOne_r);
        end
      else
      begin
        if (PE_re==1'b1) begin 
          ParityErr_r <= 1'b0; end
        if (FE_re==1'b1) begin
          FrameErr_r <= 1'b0; end
        if (BI_re==1'b1) begin
          BreakInt_r <= 1'b0; end 
      end
     end
  end




always @(posedge S_AXI_ACLK, negedge Reset)
  begin
    if (Reset == 0) begin
      OverrunErr_r <= 1'b0;
      end
    else
      if ( (empty & RbrRDn_re)||( full & rise_edge_RxIdle_r ))  begin // update at RxIdle_r rising
        // Set OverrunErr_r flag if RBR data is still not read by CPU
        OverrunErr_r <= 1'b1;
        end
      else if (OE_re) begin // clear when LSR is read
        OverrunErr_r <= 1'b0;
        end
  end



  // Receiver ready for read when data is available in RBR
  assign RxRDY = RbrDataRDY;

  // Overrun Error flag
  assign OverrunErr = OverrunErr_r;

  // Parity Error flag
  assign ParityErr = ParityErr_r;

  // Frame Error flag
  assign FrameErr = FrameErr_r;

  // Break Interrupt flag
  assign BreakInt = BreakInt_r;

endmodule

