// ===========Oooo==========================================Oooo========
// =  Copyright (C) 2014 Jinan Gowin Semiconductor Technology Co.,Ltd.
// =                     All rights reserved.
// =====================================================================
//
//  __      __      __
//  \ \    /  \    / /   [File name   ] txmitt.v
//   \ \  / /\ \  / /    [Description ] Verilog file for the UART Transmitter Module
//    \ \/ /  \ \/ /     [Timestamp   ] Tue Aug 27 15:30:00 2015
//     \  /    \  /      [version     ] 1.0
//      \/      \/       
// --------------------------------------------------------------------
// Code Revision History :
// --------------------------------------------------------------------
// Ver: | Author |Mod. Date |Changes Made:
// V1.0 | XX     |27/08/15  |Initial version
// ===========Oooo==========================================Oooo========
// 
//    <Global reset & clock>
//      Reset       : Master reset
//      Clk16X      : UART internal clock
//
//    <Register>
//      THR         : Transmitter Holding Register
//
//    <Rising edge of THR write strobe>
//      ThrWRn_re   : one Clk16X width pulse indicating rising edge of ThrWRn_r
//
//    <Transmitter output>
//      SOUT        : Transmitter serial output
//
//    <Transmitter control>
//      Databits    : "00"=5-bit, "01"=6-bit, "10"=7-bit, "11"=8-bit
//      Stopbits    : "00"=1-bit, "01"=1.5-bit(5-bit data),
//                    "10"=2-bit(6,7,8-bit data)
//      ParityEnable: 1'b0=Parity Bit Enable, 1'b1=Parity Bit Disable
//      ParityEven  : 1'b0=Even Parity Selected, 1'b1=Odd Parity Selected
//      ParityStick : 1'b0=Stick Parity Disable, 1'b1=Stick Parity Enable
//      TxBreak     : 1'b0=Disable BREAK assertion, 1'b1=Assert BREAK
//
//    <Transmitter status>
//      THRE        : THR is empty
//      TEMT        : Both THR & TSR are empty
//
// ////////////////////////////////////////////////////////////////////

module Txmitt
   (
    // Global reset & clock
    Reset       , // Master reset
    Clk16X      , // UART internal clock
    // Register THR
    THR         , // Transmitter Holding Reg
    // Rising edge of THR write strobe
    ThrWRn_re   ,   // pulse indicating rising of ThrWRn_r
    // Transmitter output
    SOUT        ,
    // Transmitter control
    DataBits    ,
    StopBits    ,
    ParityEnable,
    ParityEven  ,
    ParityStick ,
    TxBreak     ,
    // Transmitter status
    THRE        ,  //THR is empty
    TEMT      ,     // Both THR & TSR are empty
    S_AXI_ACLK  
);

    // Global reset & clock
input           S_AXI_ACLK  ;
input           Reset       ; // Master reset
input           Clk16X      ; // UART internal clock
    // Register THR
input [7:0]     THR         ; // Transmitter Holding Reg
    // Rising edge of THR write strobe
input           ThrWRn_re   ;   // pulse indicating rising of ThrWRn_r
    // Transmitter output
output          SOUT        ;
    // Transmitter control
input [1:0]     DataBits    ;
input [1:0]     StopBits    ;
input           ParityEnable;
input           ParityEven  ;
input           ParityStick ;
input           TxBreak     ;
    // Transmitter status
output          THRE        ;
output          TEMT        ;


  reg               TxOutput     ;
  reg [7:0]         TSR          ;
  reg               TxParity_r   ;
  reg               ThrEmpty     ;
  reg               TsrEmpty     ;

  reg               TxInStartState  ;
  reg               TxInShiftState  ;
  reg               TxInStopState   ;

  reg               TxInStartState1 ; // TxInStartState delayed 1 Clk16X clock
  reg               TxInShiftState1 ; // TxInShiftState delayed 1 Clk16X clock
  reg               TxInStopState1  ; // TxInStopState delayed 1 Clk16X clock

  // Transmitter Clock Enable Signals
  reg               TxClkEnA     ;

  reg [2:0]         TxCNT_r  ;
  reg [3:0]         Count_vr ;
  wire [7:0]        THRi ;
  wire              full;
  wire              empty;
  wire              fifo_wr;
  reg               fifo_rd;
  reg               fifo_rd1,fifo_rd2;
  wire              rise_edge_fifo_rd;
  // State Machine Definition
  //type state_typ is (start, shift, parity, stop_1bit, stop_2bit, stop_halfbit);
  //signal Tx_State     : state_typ;
 

 assign  fifo_wr = (ThrWRn_re) &(!full);

 always @(posedge S_AXI_ACLK) 
          if(Reset==0) 
          begin
             fifo_rd1 <= 0;
             fifo_rd2<= 0;end
          else begin
                 fifo_rd1 <= fifo_rd;
                 fifo_rd2  <= fifo_rd1;end
 assign   rise_edge_fifo_rd = fifo_rd1&(~fifo_rd2);


//---------------------------
  parameter
  IDLE=3'b111,
  start1=3'b000,
  shift=3'b001,
  parity=3'b010,
  stop_1bit=3'b011,
  stop_2bit=3'b100,
  stop_halfbit=3'b101;
  
  reg [2:0] Tx_State; 

  // Attributes for ispMACH5000VG to get higher performance
  //   These can be removed when the UART design is targeted to other devices.
  //ATTRIBUTE SYN_KEEP : integer;
  //ATTRIBUTE SYN_KEEP OF TxOutput, TxCNT_r, TSR, TxParity_r : SIGNAL IS 1;
  //ATTRIBUTE SYN_KEEP OF TsrEmpty, ThrEmpty : SIGNAL IS 1;
  //ATTRIBUTE OPT : string;
  //ATTRIBUTE OPT OF TxOutput, TxCNT_r, TSR, TxParity_r : SIGNAL IS "KEEP";
  //ATTRIBUTE OPT OF TsrEmpty, ThrEmpty : SIGNAL IS "KEEP";

////////////////////////////////////////////////////////////////////////////////
// Transmitter Finite State Machine
////////////////////////////////////////////////////////////////////////////////

fifo_syn Tx_FIFO (.datain(THR), .rd(rise_edge_fifo_rd) , .wr(fifo_wr), .rst(Reset), .clk(S_AXI_ACLK), .dataout(THRi), .full(full), .empty(empty));
  //Shift_Data_Proc: process
  always @(negedge Reset, posedge Clk16X)
  begin
    if (Reset == 0) begin
      TxCNT_r <= 'b0;
      TSR <= 'b0;
      TxOutput <= 1'b1;
      TxParity_r <= 1'b1;
      Tx_State <= IDLE;
      fifo_rd <=1'b0;
      end
    else begin
      case (Tx_State)
        IDLE : 
         if ((empty==1'b0) & (TxClkEnA==1'b1)) 
           begin           
            fifo_rd <=1'b1;
            Tx_State <= start1;
            TxOutput <= 1'b0; // start bit=1'b0
           end
         else 
            TxOutput <= 1'b1;
        
       start1 :
          if (TxClkEnA==1'b1)
            begin
            // Load data from THR to TSR
            fifo_rd <=1'b0;
            // TxParity_r initialization:
            //   set it when odd parity is selected(ParityEven=1'b0)
            //   clear it when even parity is selected(ParityEven=1'b1)
            TxParity_r <= !ParityEven ^ THRi[0];
            TxOutput <= THRi[0];
            TSR <= {1'b0,THRi[7:1]};
            TxCNT_r <= 'b0;
            Tx_State <= shift;
            end

        shift :
          if (TxClkEnA==1'b1)
            begin
             // Shift serial data out
             TSR <= {1'b0,TSR[7:1]};
             TxParity_r <= TxParity_r ^ TSR[0];
             TxOutput <= TSR[0]; // output Data bits
             TxCNT_r <= TxCNT_r + 1;
             if ((DataBits==2'b00 & TxCNT_r==4) |
                 (DataBits==2'b01 & TxCNT_r==5) |
                 (DataBits==2'b10 & TxCNT_r==6) |
                 (DataBits==2'b11 & TxCNT_r==7)) 
               if (ParityEnable==1'b0) 
                begin
                Tx_State <= stop_1bit;
                TxOutput <= 1'b1;
                end
               else
                begin
                 Tx_State <= parity;
                 if (ParityStick==1'b0) 
                   TxOutput <= TxParity_r; // output Parity bit
                 else
                   TxOutput <= ~ParityEven; // forced parity bit
                end
            end
       
        parity :
          if (TxClkEnA==1'b1) begin
            TxOutput <= 1'b1;
            Tx_State <= stop_1bit;
            end
       
        stop_1bit :
          if (TxClkEnA==1'b1) begin
            // Output 1 bit of StopBits
            if (StopBits==2'b00)  // 1 stop bit
              if(empty)
              begin
                TxOutput <= 1'b1;
                Tx_State <= IDLE;
              end
              else begin
                TxOutput <= 1'b0;
                Tx_State <= start1;
                fifo_rd <=1'b1;
              end
            else if (StopBits==2'b01)  // 1.5 stop bits(for 5-bit data only)
              Tx_State <= stop_halfbit;
            else // 2 stop bits(for 6,7,8-bit data)
              Tx_State <= stop_2bit;
            end
       
        stop_2bit :
          if (TxClkEnA==1'b1) begin
            // Output 2nd bit of 2 Stopbits
            if(empty)
            begin
              TxOutput <= 1'b1;
              Tx_State <= IDLE;
            end
            else begin
              TxOutput <= 1'b0;
              Tx_State <= start1;
              fifo_rd <=1'b1;
            end
         end
        stop_halfbit :
          if (TxClkEnA==1'b1) begin // half bit
            if(empty)
            begin
              TxOutput <= 1'b1;
              Tx_State <= IDLE;
            end
            else begin
              TxOutput <= 1'b0;
              Tx_State <= start1;
              fifo_rd <=1'b1;
            end
          end
        default :
          Tx_State <= IDLE;
      endcase
    end
  end

////////////////////////////////////////////////////////////////////////////////
// Generate TsrEmpty & ThrEmpty signals
////////////////////////////////////////////////////////////////////////////////

  // TsrEmpty : will be set whenever TSR is empty
  //TsrEmpty_Proc: process
  always @(posedge S_AXI_ACLK, negedge Reset)
  begin
    if (Reset == 0) 
      TsrEmpty <= 1'b1;
    else
      if ((TxInStopState==1'b1) & (TxInStopState1==1'b0) & empty)
        // Set TsrEmpty flag to 1'b1 when StopBit(s) is transmitted
        TsrEmpty <= 1'b1;
      else if ((TxInShiftState==1'b1) & (TxInShiftState1==1'b0)) 
        // Reset TsrEmpty flag to 1'b0 when data is transferred from THR to TSR
        TsrEmpty <= 1'b0;
  end

  // ThrEmpty : will be set whenever THR is empty
  //ThrEmpty_Proc: process
  always @(posedge S_AXI_ACLK, negedge Reset)
  begin
    if (Reset == 0) 
      ThrEmpty <= 1'b1;
    else 
      if (ThrWRn_re==1'b1) 
        // Reset ThrEmpty flag to 1'b0 when data is written into THR by CPU
        ThrEmpty <= 1'b0;
      else if ((TxInShiftState==1'b1) & (TxInShiftState1==1'b0) & empty) 
        // Set ThrEmpty flag to 1'b1 when data is transferred from THR to TSR
        // (this occurs when signal TxInShiftState is changed from 1'b0 to 1'b1)
        ThrEmpty <= 1'b1;
  end

////////////////////////////////////////////////////////////////////////////////
// Delayed signals for edge detections
////////////////////////////////////////////////////////////////////////////////
  //Delay_Signals_Proc: process
  always @(posedge Clk16X, negedge Reset)
  begin
    if (Reset == 0) begin 
      TxInStartState1 <= 1'b1;
      TxInShiftState1 <= 1'b1;
      TxInStopState1 <= 1'b1;
      end
    else begin
      // Signal for rising edge detection of signal TxInStartState
      TxInStartState1 <= TxInStartState;
      // Signal for rising edge detection of signal TxInShiftState
      TxInShiftState1 <= TxInShiftState;
      // Signal for rising edge detection of signal TxInStopState
      TxInStopState1 <= TxInStopState;
      end
  end

////////////////////////////////////////////////////////////////////////////////
// Transmitter FSM state indication signals
////////////////////////////////////////////////////////////////////////////////

  // TxInShiftState : will be set whenever transmitter is in shift state
  //TxInShiftState_Proc: process
  always @(posedge Clk16X, negedge Reset)
  begin
    if (Reset == 0) 
      TxInShiftState <= 1'b0;
    else
      if (Tx_State==shift) 
        TxInShiftState <= 1'b1;
      else
        TxInShiftState <= 1'b0;
  end

  // TxInStopState : will be set whenever transmitter is in stop_1bit state
  //TxInStopState_Proc: process
  always @(posedge Clk16X, negedge Reset)
  begin
    if (Reset == 0) 
      TxInStopState <= 1'b0;
    else
      if ( ((Tx_State==stop_1bit   ) && (StopBits == 2'b00)) || 
           ((Tx_State==stop_halfbit) && (StopBits == 2'b01)) ||
           ((Tx_State==stop_2bit   ) && (StopBits == 2'b10)) )
        TxInStopState <= 1'b1;
      else
        TxInStopState <= 1'b0;
  end

  // TxInStartState : will be set whenever transmitter is in start state
  //TxInStartState_Proc: process
  always @(posedge Clk16X, negedge Reset)
  begin
    if (Reset == 0) 
      TxInStartState <= 1'b0;
    else
      if (Tx_State==IDLE) 
        TxInStartState <= 1'b1;
      else
        TxInStartState <= 1'b0;
  end
  
////////////////////////////////////////////////////////////////////////////////
// Generate TxClkEnA/TxClkEnB signals
////////////////////////////////////////////////////////////////////////////////

  // Transmitter operates at 1/16th of the input clock frequency.
  // TxClkEnA, TxClkEnB are clock enable signals used in transmitter FSM
  //TxCLK_Proc: process
  always @(negedge Reset, posedge Clk16X)
  begin
    if (Reset == 0) begin
      Count_vr <= 4'h0;
      TxClkEnA <= 1'b0;
      end
    else begin

      if (((Count_vr==4'b0111) && (Tx_State == stop_halfbit)) || (Count_vr==4'b1111)) 
      begin
         Count_vr <= 4'h0;
         TxClkEnA <= 1'b1;
      end
      else
      begin
         TxClkEnA <= 1'b0;
         Count_vr <= Count_vr + 1;
      end
    end
  end

////////////////////////////////////////////////////////////////////////////////
// Generate THRE/TEMT flags
////////////////////////////////////////////////////////////////////////////////

  // Transmitter Holding Register Empty Indicator
  assign THRE = ThrEmpty;

  // Transmitter Empty Indicator is set to 1'b1 whenever THR & TSR are
  // both empty, & reset to 1'b1 when either THR | TSR contain a character
  assign TEMT = ((ThrEmpty==1'b1) & (TsrEmpty==1'b1)) ? 1'b1 : 1'b0;

////////////////////////////////////////////////////////////////////////////////
// Serial Data Output
////////////////////////////////////////////////////////////////////////////////

  // If Break Control bit is set to 1, the serial output is forced to Zero
  assign SOUT = (TxBreak==1'b1) ? 1'b0 : TxOutput;

endmodule
