// ===========Oooo==========================================Oooo========
// =  Copyright (C) 2014 Jinan Gowin Semiconductor Technology Co.,Ltd.
// =                     All rights reserved.
// =====================================================================
//
//  __      __      __
//  \ \    /  \    / /   [File name   ] uart_top.v
//   \ \  / /\ \  / /    [Description ] TOP Verilog file for the UART design
//    \ \/ /  \ \/ /     [Timestamp   ] Tue Aug 27 15:30:00 2015
//     \  /    \  /      [version     ] 1.0
//      \/      \/       
// --------------------------------------------------------------------
// Code Revision History :
// --------------------------------------------------------------------
// Ver: | Author |Mod. Date |Changes Made:
// V1.0 | XX     |27/08/15  |Initial version
// ===========Oooo==========================================Oooo========
//                 
//     <Global reset & clock>
//      MR     : Master reset
//      MCLK   : Master clock
//
//    <Processor interface>
//      A      : Address bus
//      DIN    : Data bus input
//      DOUT   : Data bus output
//      ADSn   : Address strobe
//      CS     : Chip Select
//      RDn    : Read
//      WRn    : Write
//      DDIS   : Driver disable
//      INTR   : Interrupt
//
//    <Receiver interface>
//      SIN    : Receiver serial input
//      RxRDYn : Receiver ready, low active when RBR data is available
//
//    <Transmitter interface>
//      SOUT   : Transmitter serial output
//      TxRDYn : Transmitter ready, low active when THR is empty
//
//    <Modem interface>
//      DCDn   : Data Carrier Detect
//      CTSn   : Clear To Send
//      DSRn   : Data Set Ready
//      RIn    : Ring Indicator
//      DTRn   : Data Terminal Ready
//      RTSn   : Request To Send
//
// ////////////////////////////////////////////////////////////////////

`define C_S_AXI_DATA_WIDTH 32
`define C_S_AXI_ADDR_WIDTH 32

module Uart_top
    (
    S_AXI_ACLK     ,
    S_AXI_ARESETN  ,
    S_AXI_AWADDR   ,
    S_AXI_AWVALID  ,  
    S_AXI_AWREADY  ,
    S_AXI_WDATA    ,
    S_AXI_WSTRB    ,
    S_AXI_WVALID   ,
    S_AXI_WREADY   ,
    S_AXI_BRESP    ,
    S_AXI_BVALID   ,
    S_AXI_BREADY   ,
    S_AXI_ARADDR   ,
    S_AXI_ARVALID  ,
    S_AXI_ARREADY  ,
    S_AXI_RDATA    ,
    S_AXI_RRESP    ,
    S_AXI_RVALID   ,
    S_AXI_RREADY   ,

    // Processor interface
    DDIS   , // Driver disable
    INTR   , // Interrupt

    // Receiver interface
    SIN    , // Receiver serial input
    RxRDYn , // Receiver ready

    // Transmitter interface
    SOUT   , // Transmitter serial output
    TxRDYn , // Transmitter ready

    // Modem interface
    DCDn   , // Data Carrier Detect
    CTSn   , // Clear To Send
    DSRn   , // Data Set Ready
    RIn    , // Ring Indicator
    DTRn   , // Data Terminal Ready
    RTSn     // Request To Send
  );
 parameter  fsysclk = 50_000_000 ;
 parameter  baud    = 115_200    ;

 input                                S_AXI_ACLK;
 input                                S_AXI_ARESETN;
 input   [`C_S_AXI_ADDR_WIDTH - 1:0]  S_AXI_AWADDR;
 input                                S_AXI_AWVALID;
 output                               S_AXI_AWREADY;
 input   [`C_S_AXI_DATA_WIDTH-1:0]    S_AXI_WDATA;
 input   [`C_S_AXI_DATA_WIDTH/8-1:0]  S_AXI_WSTRB;
 input                                S_AXI_WVALID;
 output                               S_AXI_WREADY;
 output  [1:0]                        S_AXI_BRESP;
 output                               S_AXI_BVALID;
 input                                S_AXI_BREADY;
 input   [`C_S_AXI_ADDR_WIDTH - 1:0]  S_AXI_ARADDR;
 input                                S_AXI_ARVALID;
 output                               S_AXI_ARREADY;
 output  [`C_S_AXI_DATA_WIDTH-1:0]    S_AXI_RDATA;
 output  [1:0]                        S_AXI_RRESP;
 output                               S_AXI_RVALID;
 input                                S_AXI_RREADY;

 // Processor interface
 output            DDIS   ; // Driver disable
 output            INTR   ; // Interrupt

    // Receiver interface
 input             SIN    ; // Receiver serial input
 output            RxRDYn ; // Receiver ready

    // Transmitter interface
 output            SOUT   ; // Transmitter serial output
 output            TxRDYn ; // Transmitter ready

    // Modem interface
 input             DCDn   ; // Data Carrier Detect
 input             CTSn   ; // Clear To Send
 input             DSRn   ; // Data Set Ready
 input             RIn    ; // Ring Indicator
 output            DTRn   ; // Data Terminal Ready
 output            RTSn   ;  // Request To Send/

  
  wire       rxclk       ;

  wire [7:0] RBR         ;
  wire [7:0] THR         ;
  wire [7:0] MSR         ;
  wire [1:0] MCR         ;

  wire [1:0] DataBits    ;
  wire [1:0] StopBits    ;
  wire       ParityEnable;
  wire       ParityEven  ;
  wire       ParityStick ;
  wire       TxBreak     ;

  wire       ThrWRn_re   ;
  wire       RbrRDn_re   ;
  wire       LsrRDn_re   ;
  wire       MsrRDn_re   ;

  wire       RxRDY       ;
  wire       ParityErr   ;
  wire       FrameErr    ;
  wire       OverrunErr  ;
  wire       BreakInt    ;

  wire       THRE        ;
  wire       TEMT        ;

  wire       OE_re     ;   //Clear OverrunErr
  wire       PE_re     ;   //Clear ParityErr
  wire       FE_re     ;   //Clear FrameErr
  wire       BI_re     ;   //Clear BreakInt



  axi_lite_slave  u_axi_lite_slave(
    .S_AXI_ACLK     (S_AXI_ACLK),
    .S_AXI_ARESETN  (S_AXI_ARESETN),
    .S_AXI_AWADDR   (S_AXI_AWADDR),
    .S_AXI_AWVALID  (S_AXI_AWVALID),  
    .S_AXI_AWREADY  (S_AXI_AWREADY),
    .S_AXI_WDATA    (S_AXI_WDATA),
    .S_AXI_WSTRB    (S_AXI_WSTRB),
    .S_AXI_WVALID   (S_AXI_WVALID),
    .S_AXI_WREADY   (S_AXI_WREADY),
    .S_AXI_BRESP    (S_AXI_BRESP),
    .S_AXI_BVALID   (S_AXI_BVALID),
    .S_AXI_BREADY   (S_AXI_BREADY),
    .S_AXI_ARADDR   (S_AXI_ARADDR),
    .S_AXI_ARVALID  (S_AXI_ARVALID),
    .S_AXI_ARREADY  (S_AXI_ARREADY),
    .S_AXI_RDATA    (S_AXI_RDATA),
    .S_AXI_RRESP    (S_AXI_RRESP),
    .S_AXI_RVALID   (S_AXI_RVALID),
    .S_AXI_RREADY   (S_AXI_RREADY),
  
    .Clk16X       (rxclk), 
    .DDIS         (DDIS),
    .INTR         (INTR),
    .RBR          (RBR),
    .THR          (THR),
    .MSR          (MSR),
    .MCR          (MCR),
    .RbrRDn_re    (RbrRDn_re),
    .ThrWRn_re    (ThrWRn_re),
    .LsrRDn_re    (LsrRDn_re),
    .MsrRDn_re    (MsrRDn_re),
    .DataBits     (DataBits),
    .StopBits     (StopBits),
    .ParityEnable (ParityEnable),
    .ParityEven   (ParityEven),
    .ParityStick  (ParityStick),
    .TxBreak      (TxBreak),
    .RxRDY        (RxRDY),
    .OverrunErr   (OverrunErr),
    .ParityErr    (ParityErr),
    .FrameErr     (FrameErr),
    .BreakInt     (BreakInt),
    .THRE         (THRE),
    .TEMT         (TEMT),
    .OE_re        (OE_re),
    .PE_re        (PE_re),
    .FE_re        (FE_re),
    .BI_re        (BI_re)

  ); 

 
  Modem U_Modem(
    // Global reset & clock
    .Reset        (S_AXI_ARESETN),
    .S_AXI_ACLK   (S_AXI_ACLK),
    // Registers
    .MSR          (MSR),
    .MCR          (MCR),
    // Rising Edge of MSR Read Strobe
    .MsrRDn_re    (MsrRDn_re),
    // Modem interface
    .DCDn         (DCDn),
    .CTSn         (CTSn),
    .DSRn         (DSRn),
    .RIn          (RIn),
    .DTRn         (DTRn),
    .RTSn         (RTSn)
  );

  Rxcver U_Rxcver(
    // Global reset & clock
    .S_AXI_ACLK   (S_AXI_ACLK),
    .Reset        (S_AXI_ARESETN),
    .Clk16X       (rxclk),
    // Register RBR
    .RBR          (RBR),
    // Rising edge of RBR), LSR read strobes
    .RbrRDn_re    (RbrRDn_re),
    .LsrRDn_re    (LsrRDn_re),
    // Receiver input
    .SIN          (SIN),
    // Receiver control
    .DataBits     (DataBits),
    .ParityEnable (ParityEnable),
    .ParityEven   (ParityEven),
    .ParityStick  (ParityStick),
    // Receiver status
    .RxRDY        (RxRDY),
    .OverrunErr   (OverrunErr),
    .ParityErr    (ParityErr),
    .FrameErr     (FrameErr),
    .BreakInt     (BreakInt),
    .OE_re        (OE_re),
    .PE_re        (PE_re),
    .FE_re        (FE_re),
    .BI_re        (BI_re)

  );

  Txmitt U_Txmitt(
    // Global reset & clock
    .Reset        (S_AXI_ARESETN),
    .Clk16X       (rxclk),
    // Register THR
    .THR          (THR),
    // Rising edge of THR write strobe
    .ThrWRn_re    (ThrWRn_re),
    // Transmitter control
    .DataBits     (DataBits),
    .StopBits     (StopBits),
    .ParityEnable (ParityEnable),
    .ParityEven   (ParityEven),
    .ParityStick  (ParityStick),
    .TxBreak      (TxBreak),
    // Transmitter output
    .SOUT         (SOUT),
    // Transmitter status
    .THRE         (THRE),
    .TEMT         (TEMT),
    .S_AXI_ACLK   (S_AXI_ACLK)
  );

 baudset #(
   .fsysclk       (fsysclk),
   .baud          (baud)
  )u_baudset      (
   .rst           (S_AXI_ARESETN),
   .sysclk        (S_AXI_ACLK),
   .rxclk         (rxclk)
  );
        


  // TxRDYn, RxRDYn is low active output
  assign TxRDYn = ~THRE;
  assign RxRDYn = ~RxRDY;

endmodule
